<?php
/*
 * Copyright (c) Nosial 2022-2023, all rights reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
 *  associated documentation files (the "Software"), to deal in the Software without restriction, including without
 *  limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 *  Software, and to permit persons to whom the Software is furnished to do so, subject to the following
 *  conditions:
 *
 *  The above copyright notice and this permission notice shall be included in all copies or substantial portions
 *  of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 *  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 *  PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 *  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 *  OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *  DEALINGS IN THE SOFTWARE.
 *
 */

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace ncc\Objects\ComposerJson;

    class Author
    {
        /**
         * The author's name. Usually their real name
         *
         * @var string|null
         */
        public $Name;

        /**
         * The author's email address
         *
         * @var string|null
         */
        public $Email;

        /**
         * URL to the author's website
         *
         * @var string|null
         */
        public $Homepage;

        /**
         * The author's role in the project (eg. developer or translator)
         *
         * @var string|null
         */
        public $Role;

        /**
         * Returns an array representation of the object
         *
         * @return array
         */
        public function toArray(): array
        {
            return [
                'name' => $this->Name,
                'email' => $this->Email,
                'homepage' => $this->Homepage,
                'role' => $this->Role
            ];
        }

        /**
         * Constructs object from an array representation
         *
         * @param array $data
         * @return Author
         */
        public static function fromArray(array $data): self
        {
            $object = new self();

            if(isset($data['name']))
                $object->Name = $data['name'];

            if(isset($data['email']))
                $object->Email = $data['email'];

            if(isset($data['homepage']))
                $object->Homepage = $data['homepage'];

            if(isset($data['role']))
                $object->Role = $data['role'];

            return $object;
        }
    }